﻿/*
 * Copyright (c) 2013 Cem Dervis
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software
 * is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
*/

using System;
using System.ComponentModel;
using System.Globalization;
using System.Windows.Forms;

namespace ImageToaster
{
    public sealed class NumericTextBox : TextBox
    {
        private int mMinimum;
        private int mMaximum;

        public NumericTextBox()
        {
            mMinimum = 0;
            mMaximum = 100;

            this.Value = 100;
        }

        protected override void OnCreateControl()
        {
            if (!DesignMode)
            {
                this.KeyPress += NumericTextBox_KeyPress;
            }

            base.OnCreateControl();
        }

        void NumericTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = (!char.IsDigit(e.KeyChar)) && e.KeyChar != '\b';
        }

        [Browsable(false)]
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        public new string Text
        {
            get { return base.Text; }
            set { base.Text = value; }
        }

        protected override void OnTextChanged(EventArgs e)
        {
            base.OnTextChanged(e);

            int val = 0;
            if (!int.TryParse(this.Text, out val))
                val = Minimum;

            val = Math.Min(mMaximum, Math.Max(mMinimum, val));

            Text = val.ToString(CultureInfo.InvariantCulture);

            if (val.ToString().Length == mMaximum.ToString().Length)
                Select(Text.Length, 0);
        }

        public int Value
        {
            get
            {
                int val = 0;
                if (!int.TryParse(this.Text, out val))
                    val = Minimum;

                return val;
            }
            set
            {
                int val = Math.Min(mMaximum, Math.Max(mMinimum, value));
                this.Text = val.ToString(CultureInfo.InvariantCulture);
            }
        }

        public int Minimum
        {
            get { return mMinimum; }
            set
            {
                if (value >= mMaximum)
                    throw new IndexOutOfRangeException("Minimum");

                mMinimum = value;
            }
        }

        public int Maximum
        {
            get { return mMaximum; }
            set
            {
                if (value < mMinimum)
                    throw new IndexOutOfRangeException("Maximum");

                mMaximum = value;
            }
        }

    }
}
