﻿/*
 * Copyright (c) 2013 Cem Dervis
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software
 * is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;

namespace ImageToaster
{
    public partial class ResizerForm : Form
    {
        #region Fields

        // Original image size.
        private Size mOriginalSize;

        // Selected files.
        private readonly List<string> mFiles;

        // Current image memory stream; needed for Image.Save.
        private MemoryStream mCurrentImgStream;

        #endregion

        #region Initialization

        public ResizerForm()
        {
            InitializeComponent();
        }

        public ResizerForm(List<string> files)
            : this()
        {
            mFiles = files;
        }

        #endregion

        #region Private Methods

        private void DisposeCurrentImage()
        {
            if (picBox.Image != null)
            {
                picBox.Dispose();
                picBox.Image = null;
            }

            if (mCurrentImgStream != null)
            {
                mCurrentImgStream.Dispose();
                mCurrentImgStream = null;
            }
        }

        private void SetImage(string file)
        {
            ResetControlsState();

            lblImgFile.Text = Path.GetFileName(file);

            Image img = ImageLoader.Load(file, out mCurrentImgStream);

            picBox.Image = img;
            mOriginalSize = img.Size;

            numWidth.Value = img.Width;
            numHeight.Value = img.Height;
        }

        private void ResizeImage(Image image, string srcFile)
        {
            string dstFile = srcFile;
            string ext = Path.GetExtension(srcFile).ToLowerInvariant();

            if (!chkOverwrite.Checked)
            {
                string withoutExt = Path.ChangeExtension(srcFile, null);
                dstFile = withoutExt + "_resized" + ext;
            }

            var resized = (Image)(new Bitmap(image, new Size(numWidth.Value, numHeight.Value)));

            resized.Save(dstFile, ImageFormatHelper.GetImageFormatFromExtension(ext));

            resized.Dispose();
        }

        private void UpdatePixelValues()
        {
            bool useProc = radPercentage.Checked;

            if (useProc)
            {
                int proc = int.Parse(cmbPercent.SelectedItem.ToString());

                float div = 100.0f/proc;

                numWidth.Value = (int)Math.Max(1, (float)mOriginalSize.Width/div);
                numHeight.Value = (int)Math.Max(1, (float)mOriginalSize.Height / div);
            }
        }

        private void ResetControlsState()
        {
            radPercentage.Checked = true;
            cmbPercent.SelectedIndex = 0;
            cmbPercent.Enabled = true;
        }

        private void UpdateProportionalWidth()
        {
            if (chkProportional.Checked)
            {
                float prop = ((float)mOriginalSize.Width / (float)mOriginalSize.Height);
                prop = Math.Max(1, prop);

                int w = (int)((float)numHeight.Value * prop);
                w = Math.Max(numWidth.Minimum, Math.Min(numWidth.Maximum, w));
                numWidth.Value = w;
            }
        }

        private void UpdateProportionalHeight()
        {
            if (chkProportional.Checked)
            {
                float prop = ((float)mOriginalSize.Width / (float)mOriginalSize.Height);
                prop = Math.Max(1, prop);

                int h = (int)((float)numWidth.Value / prop);
                h = Math.Max(numWidth.Minimum, Math.Min(numWidth.Maximum, h));
                numHeight.Value = h;
            }
        }

        #endregion

        #region Events

        private void ResizerForm_Load(object sender, EventArgs e)
        {
            SetImage(mFiles[0]);
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            if (!File.Exists(mFiles[0]))
            {
                MessageBox.Show(this,
                    string.Format("Image {0} does not exist anymore.", Path.GetFileName(mFiles[0])), "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            else
                ResizeImage(picBox.Image, mFiles[0]);

            // Remove the file in any case.
            mFiles.RemoveAt(0);

            if (mFiles.Count > 0)
            {
                SetImage(mFiles[0]);
            }
            else
            {
                this.DialogResult = DialogResult.OK;
            }
        }

        private void btnSkip_Click(object sender, EventArgs e)
        {
            if (mFiles.Count > 0)
            {
                mFiles.RemoveAt(0);
                DisposeCurrentImage();
                ResetControlsState();
            }

            if (mFiles.Count == 0)
                this.DialogResult = DialogResult.OK;
        }

        private void radPercentage_CheckedChanged(object sender, EventArgs e)
        {
            cmbPercent.Enabled = radPercentage.Checked;
        }

        private void radAbsSize_CheckedChanged(object sender, EventArgs e)
        {
            bool enabled = radAbsSize.Checked;
            numWidth.Enabled = enabled;
            numHeight.Enabled = enabled;
            chkProportional.Enabled = enabled;
        }

        private void radFbCoverPhoto_CheckedChanged(object sender, EventArgs e)
        {
            var size = new Size(851, 351);
            numWidth.Value = size.Width;
            numHeight.Value = size.Height;
            radAbsSize.Checked = true;
        }

        private void radFbProfilePic_CheckedChanged(object sender, EventArgs e)
        {
            var size = new Size(180, 180);
            numWidth.Value = size.Width;
            numHeight.Value = size.Height;
            radAbsSize.Checked = true;
        }

        private void lblImgFile_MouseEnter(object sender, EventArgs e)
        {
            lblImgFile.ForeColor = Color.Blue;
        }

        private void lblImgFile_MouseLeave(object sender, EventArgs e)
        {
            lblImgFile.ForeColor = Color.Black;
        }

        private void lblImgFile_Click(object sender, EventArgs e)
        {
            if (File.Exists(mFiles[0]))
                Process.Start(mFiles[0]);
        }

        private void chkProportional_CheckedChanged(object sender, EventArgs e)
        {
            if (chkProportional.Checked)
            {
                numWidth.Value = numWidth.Value;
                numHeight.Value = numHeight.Value;
            }
        }

        private void numWidth_TextChanged(object sender, EventArgs e)
        {
            UpdateProportionalHeight();
        }

        private void numHeight_TextChanged(object sender, EventArgs e)
        {
            UpdateProportionalWidth();
        }

        private void cmbPercent_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdatePixelValues();
        }

        private void ResizerForm_Paint(object sender, PaintEventArgs e)
        {
            var g = e.Graphics;

            var gradBotColor = Color.FromArgb(220, 220, 220);
            var gradTopColor = Color.White;

            using (var brush = new LinearGradientBrush(ClientRectangle, gradTopColor, gradBotColor, 90))
            {
                g.FillRectangle(brush, ClientRectangle);
            }
        }

        #endregion
    }
}
